/*-------------------------------------------------------------------------------------------------
  File Name:  winbgi.c

  Purpose:    RCsimWin - 
              Map the required subset of Borland Graphics Interface into MS Windows GDI.

  Author:     David B. Ewing
              codetoad@rocketmail.com

  History:    07/04/98  Created

  
             Copyright (c) 1998  David B. Ewing 
    All rights reserved.  May be copied and distributed freely,
    provided this copyright notice is preserved.

    RCsim - Copyright (c) 1995 Mikko Oksalahti and Harri Rautio
-------------------------------------------------------------------------------------------------*/

#define STRICT

#include <windows.h>
#include "graphics.h"

extern HDC mainDC;
extern HDC drawDC;
extern HWND hMainWindow;
extern BOOL doPageFlipping;

static HBRUSH currentBrush;
static HBRUSH backgroundBrush;
static HPEN currentPen;
static HFONT hFont;
static HBITMAP screenBmp;


/* Map BGI colors to COLORREFs */
static COLORREF crColorMap[] = {
    PALETTERGB (0,0,0),// BLACK
    PALETTERGB (0,0,255),// BLUE
    PALETTERGB (0,255,0),// GREEN
    PALETTERGB (0,255,255),// CYAN
    PALETTERGB (255,0,0),// RED
    PALETTERGB (255,0,255),// MAGENTA
    PALETTERGB (128,64,0),// BROWN
    PALETTERGB (200,200,200),// LIGHTGRAY
    PALETTERGB (82,82,82),// DARKGRAY
    PALETTERGB (0,0,255),// LIGHTBLUE
    PALETTERGB (0,127,0),// LIGHTGREEN
    PALETTERGB (0,127,127),// LIGHTCYAN
    PALETTERGB (127,0,0),// LIGHTRED
    PALETTERGB (127,0,127),// LIGHTMAGENTA
    PALETTERGB (255,255,0),// YELLOW
    PALETTERGB (255,255,255) // WHITE
};

static HBRUSH hbrColorMap[sizeof (crColorMap) / sizeof (COLORREF)];
static HPEN hpenColorMap[sizeof (crColorMap) / sizeof (COLORREF)];

RECT deviceClientRect;
const RECT logicalClientRect = {0,0,640,350}; /* EGAHI */
void InitDrawDC (HDC dc);

//-------------------------------------------------------------------------------------------------
void setbkcolor(int color)
{
  backgroundBrush = hbrColorMap[color];
}

//-------------------------------------------------------------------------------------------------
void bar(int left, int top, int right, int bottom)
{
  Rectangle (drawDC, left, top, right, bottom);
}

//-------------------------------------------------------------------------------------------------
void cleardevice(void)
{
  FillRect(drawDC, &logicalClientRect, backgroundBrush);
}

//-------------------------------------------------------------------------------------------------
void fillpoly(int numpoints, const int *polypoints)
{
  Polygon (drawDC, (POINT *)polypoints, numpoints);
}

//-------------------------------------------------------------------------------------------------
int  graphresult(void)
{
  return grOk;
}

//-------------------------------------------------------------------------------------------------
void initgraph(int *graphdriver, int *graphmode, const char *pathtodriver)
{
  int i;

  /* Create brushes and pens mapped to BGI colors */
  for (i = 0; i < sizeof (crColorMap) / sizeof (COLORREF); i++)  {
    hbrColorMap[i]  = CreateSolidBrush(crColorMap[i]);
    hpenColorMap[i] = CreatePen(PS_SOLID, 0, crColorMap[i]);
  }

  GetClientRect(hMainWindow, &deviceClientRect); 
  SetMapMode(mainDC, MM_ANISOTROPIC);
  
  /* Set logical coordinate extent */
  SetWindowExtEx(mainDC, logicalClientRect.right, logicalClientRect.bottom, NULL);
  
  /* Set device coordinate (pixel) extent */
  SetViewportExtEx(mainDC, deviceClientRect.right, deviceClientRect.bottom, NULL); 

  /* Initialize the DC we'll draw to */
  InitDrawDC (mainDC);

  /* We need a groovy font */
  hFont = CreateFont (12,0,0,0,0,0,0,0,0,0,0,0,0,"Arial");
  SelectObject (drawDC, hFont);

  /* Text background */
  SetBkMode (drawDC, TRANSPARENT);
}

//-------------------------------------------------------------------------------------------------
void line(int x1, int y1, int x2, int y2)
{
  POINT points[] = {{x1, y1}, {x2, y2}};
  Polyline (drawDC, points, 2);
}

//-------------------------------------------------------------------------------------------------
void setcolor(int color)
{
  currentBrush = hbrColorMap[color];
  currentPen   = hpenColorMap[color];

  /* Set current pen, brush, and text color */
  SelectObject (drawDC, currentBrush);
  SelectObject (drawDC, currentPen);
  SetTextColor (drawDC, crColorMap[color]);
}

//-------------------------------------------------------------------------------------------------
void setfillstyle(int pattern, int color)
{
  currentBrush = hbrColorMap[color];
  SelectObject (drawDC, currentBrush);
}

//-------------------------------------------------------------------------------------------------
void setactivepage(int page)
  /* If page-flipping is activated, we'll do a bit block transfer */
{
  if (!doPageFlipping)
    return;

  /* The BLT is MUCH faster in MM_TEXT (pixel) mapping mode */
  SetMapMode(drawDC, MM_TEXT);
  
  BitBlt (mainDC,  // handle of destination device context 
          0,      // x-coordinate of destination rectangle's upper-left corner
          0,      // y-coordinate of destination rectangle's upper-left corner
          deviceClientRect.right,   // width of destination rectangle 
          deviceClientRect.bottom,  // height of destination rectangle 
          drawDC,  // handle of source device context 
          0,      // x-coordinate of source rectangle's upper-left corner  
          0,      // y-coordinate of source rectangle's upper-left corner
          SRCCOPY // raster operation code
  ); 

  /* Restore drawing mapping mode */
  SetMapMode(drawDC, MM_ANISOTROPIC);

  /* Set logical coordinate extent */
  SetWindowExtEx(drawDC, logicalClientRect.right, logicalClientRect.bottom, NULL);
  
  /* Set device coordinate (pixel) extent */
  SetViewportExtEx(drawDC, deviceClientRect.right, deviceClientRect.bottom, NULL); 
}

//-------------------------------------------------------------------------------------------------
void setvisualpage(int page)
{
}

//-------------------------------------------------------------------------------------------------
void outtextxy(int x, int y, const char  *textstring)
{
  TextOut(drawDC, x, y, textstring, strlen (textstring));
}

//-------------------------------------------------------------------------------------------------
void InitDrawDC (HDC mainDC)
  /* Initialize a memory DC for drawing (page-flip to mainDC) */
{
  if (doPageFlipping)  {
    int nWidth  = GetSystemMetrics (SM_CXFULLSCREEN);
    int nHeight = GetSystemMetrics (SM_CYFULLSCREEN);
    drawDC = CreateCompatibleDC (mainDC);

    /* Size DC to max screen size */
    screenBmp = CreateCompatibleBitmap (mainDC, nWidth, nHeight);
    SelectObject (drawDC, screenBmp);

    /* We'll draw to drawDC, and copy (BLT) pixels to mainDC */
    SetMapMode(mainDC, MM_TEXT);
  }
  else  {
    /* Draw directly to screen DC */
    drawDC = mainDC;
  }
}

//-------------------------------------------------------------------------------------------------
void CleanupBGI (void)
  /* Free associated system resources */
{
  int i;

  if (!drawDC)  {
    return;
  }

  /* Delete brushes and pens mapped to BGI colors */
  for (i = 0; i < sizeof (crColorMap) / sizeof (COLORREF); i++)  {
    DeleteObject (hbrColorMap[i]);
    DeleteObject (hpenColorMap[i]);
  }

  DeleteObject (drawDC);
  DeleteObject (screenBmp);
  DeleteObject (hFont);
}

//-------------------------------------------------------------------------------------------------

