/*-------------------------------------------------------------------------------------------------
  File Name:  wincore.c

  Purpose:    RCsimWin - Win32 port of RCsim

  Author   :  David B. Ewing
              codetoad@rocketmail.com

  History:    07/04/98  Created

             Copyright (c) 1998  David B. Ewing 
    All rights reserved.  May be copied and distributed freely,
    provided this copyright notice is preserved.

    RCsim - Copyright (c) 1995 Mikko Oksalahti and Harri Rautio
-------------------------------------------------------------------------------------------------*/

#define STRICT

#include <windows.h>
#include <windowsx.h>
#include <string.h>
#pragma hdrstop
#pragma warning( disable : 4101 )
#include "resource.h"
#include "joystick.h"
#include "graphics.h"

#define Wincore_DefProc DefWindowProc
#define IDT_TIMER_SPLASH  1
#define WM_STARTAPP  (WM_USER + 1)

#define SPLASH_DELAY  1000       /* Splash screen duration (ms) */
#define TIMER_REZ     1          /* Multimedia timer resolution (ms) */

HWND hMainWindow;
HINSTANCE hInstance;
HDC mainDC;      /* Main window sports a private DC */
HDC drawDC = 0;
BOOL doPageFlipping;
static UINT wTimerID;
static BOOL isPaused = FALSE;

//-------------------------------------------------------------------------------------------------
//Application core
BOOL InitApplication(HINSTANCE);
BOOL InitInstance(HINSTANCE, int);
LRESULT CALLBACK WndProc(HWND, UINT, WPARAM, LPARAM);

//Dialog procs
LRESULT CALLBACK About(HWND, UINT, WPARAM, LPARAM);

//Message handlers
BOOL Wincore_OnCreate (HWND hwnd, CREATESTRUCT FAR *lpCreateStruct);
void Wincore_OnDestroy (HWND hwnd);
void Wincore_OnClose(HWND hwnd);
void Wincore_OnCommand (HWND hwnd, int id, HWND hwndCtl, UINT codeNotify);
void Wincore_OnStartApp (HWND hwnd);
void Wincore_OnSize(HWND hwnd, UINT state, int cx, int cy);
void Wincore_OnChar(HWND hwnd, TCHAR ch, int cRepeat);
BOOL Wincore_OnEraseBkgnd(HWND hwnd, HDC hdc);

//-------------------------------------------------------------------------------------------------
int APIENTRY WinMain(HINSTANCE hInstance, HINSTANCE hPrevInstance, LPSTR lpCmdLine, int nCmdShow)
{
  MSG msg;

  //Perform instance initialization
  if (!hPrevInstance) 
    if (!InitApplication(hInstance))
      return (FALSE);

  //Perform application initialization
  if (!InitInstance(hInstance, nCmdShow))
    return (FALSE);

  // Main message loop:
  while (GetMessage (&msg, NULL, 0, 0)) {
    TranslateMessage(&msg);
    DispatchMessage(&msg);
  }

  return msg.wParam;
}

//-------------------------------------------------------------------------------------------------
static BOOL InitApplication(HINSTANCE hInstance)
{
  WNDCLASSEX  wc;
  HWND      hwnd;

  // Fill in window class structure with parameters that describe the main window.
  wc.style         = CS_HREDRAW | CS_VREDRAW | CS_OWNDC;
  wc.lpfnWndProc   = (WNDPROC)WndProc;
  wc.cbClsExtra    = 0;
  wc.cbWndExtra    = 0;
  wc.hInstance     = hInstance;
  wc.hIcon         = LoadIcon (hInstance, MAKEINTRESOURCE (BIG_ICO));
  wc.hCursor       = LoadCursor(NULL, IDC_ARROW);
  wc.hbrBackground = (HBRUSH)(COLOR_WINDOW+1);
  wc.lpszMenuName  = MAKEINTRESOURCE (MENU1);
  wc.lpszClassName = "RCsimWin";
  wc.cbSize        = sizeof(WNDCLASSEX);
  wc.hIconSm       = LoadIcon(hInstance, MAKEINTRESOURCE (SM_ICO));

  return RegisterClassEx(&wc);   
}

//-------------------------------------------------------------------------------------------------
static BOOL InitInstance (HINSTANCE hInst, int nCmdShow)
{
  hInstance = hInst;
  hMainWindow = CreateWindow("RCsimWin", "RCsimWin", WS_OVERLAPPEDWINDOW, CW_USEDEFAULT, 0, CW_USEDEFAULT, 0,
                             NULL, NULL, hInstance, NULL);

  if (!hMainWindow) {
    return (FALSE);
  }

  ShowWindow(hMainWindow, nCmdShow);
  UpdateWindow(hMainWindow);

  PostMessage (hMainWindow, WM_STARTAPP, 0, 0);

  return TRUE;
}

//-------------------------------------------------------------------------------------------------
static LRESULT CALLBACK WndProc(HWND hwnd, UINT message, WPARAM wParam, LPARAM lParam)
{
  switch (message) { 
    HANDLE_MSG(hwnd, WM_CREATE, Wincore_OnCreate);
    HANDLE_MSG(hwnd, WM_DESTROY, Wincore_OnDestroy);
    HANDLE_MSG(hwnd, WM_COMMAND, Wincore_OnCommand);
    HANDLE_MSG(hwnd, WM_CLOSE, Wincore_OnClose);
    HANDLE_MSG(hwnd, WM_SIZE, Wincore_OnSize);
    HANDLE_MSG(hwnd, WM_CHAR, Wincore_OnChar);
    HANDLE_MSG(hwnd, WM_ERASEBKGND, Wincore_OnEraseBkgnd);
    case WM_STARTAPP:
      Wincore_OnStartApp (hwnd);
      return TRUE;
    default:
      return Wincore_DefProc(hwnd, message, wParam, lParam);
  }
}

//-------------------------------------------------------------------------------------------------
static BOOL Wincore_OnEraseBkgnd(HWND hwnd, HDC hdc)
{
  return TRUE;  /* We'll handle erasing background */
}

//-------------------------------------------------------------------------------------------------
static void Wincore_OnChar(HWND hwnd, TCHAR ch, int cRepeat)
{
  extern void ReadKeyboardControls (char c);
 
  /* Minimize/Pause on ESC keypress */
  if (ch == VK_ESCAPE)  {
    ShowWindow (hwnd, SW_MINIMIZE);
  }

  ReadKeyboardControls ((char)tolower(ch));
}

//-------------------------------------------------------------------------------------------------
static void Wincore_OnSize(HWND hwnd, UINT state, int cx, int cy)
{
  extern RECT deviceClientRect;
  extern const RECT logicalClientRect;

  if (!drawDC)  {
    return;
  }

  if (state == SIZE_MINIMIZED)  {
    isPaused = TRUE;
    return;
  }
  else  {
    isPaused = FALSE;
  }

  GetClientRect(hMainWindow, &deviceClientRect); 

  /* Set logical coordinate extent */
  SetWindowExtEx(drawDC, logicalClientRect.right, logicalClientRect.bottom, NULL);
  
  /* Set device coordinate (pixel) extent */
  SetViewportExtEx(drawDC, deviceClientRect.right, deviceClientRect.bottom, NULL); 
}

//-------------------------------------------------------------------------------------------------
static void CALLBACK Tmr_handler (UINT uID, UINT uMsg, DWORD nMove, DWORD dw1, DWORD dw2)
{
  extern void RCsim_poll (int nMove);
  
  if (isPaused)
    return;

  RCsim_poll (nMove);
}

//-------------------------------------------------------------------------------------------------
static void Wincore_OnStartApp (HWND hwnd)
{
  extern int RCsim_init (char *arg);
  extern int skyColor, earthColor, shadowColor;
  extern UINT xFullScale;
  extern UINT yFullScale;
  char iniFileName[1024];
  char sbuf[80];
  char *pathEnd;
  int nModelUpdatesPerFrame;
  int msFrameRate;

  OutputDebugString ("Start Application\n");

  /* Get our private DC (window style has CS_OWNDC set) */
  mainDC = GetDC (hwnd);

  /* Read parameters from private ini file */
  GetModuleFileName (hInstance, iniFileName, sizeof (iniFileName));
  pathEnd = strrchr (iniFileName, '\\');
  if (pathEnd)  {
    *(pathEnd + 1) = 0;
  }
  strcat (iniFileName, "RCsimWin.ini");

  GetPrivateProfileString ("config", "weather", "CLOUDY", sbuf, sizeof(sbuf), iniFileName);
  if (stricmp (sbuf, "CLOUDY"))  {
    /* If it's not cloudy, it must be sunny! */
    skyColor    = LIGHTBLUE;
    earthColor  = LIGHTGREEN;
    shadowColor = BLACK;
  }

  GetPrivateProfileString ("config", "page_flip", "NO", sbuf, sizeof(sbuf), iniFileName);
  doPageFlipping = stricmp (sbuf, "NO");

  GetPrivateProfileString ("config", "plane_file", "trner486.pla", sbuf, sizeof(sbuf), iniFileName);
  RCsim_init (sbuf);

  nModelUpdatesPerFrame = GetPrivateProfileInt ("config", "moves_per_frame", 1, iniFileName);
  msFrameRate = GetPrivateProfileInt ("config", "frame_period_ms", 30, iniFileName);
  
  xFullScale = GetPrivateProfileInt ("config", "joy_x_range", xFullScale, iniFileName);
  yFullScale = GetPrivateProfileInt ("config", "joy_y_range", yFullScale, iniFileName);

  init_joystick();

  timeBeginPeriod (TIMER_REZ); // resolution (ms)
  wTimerID = timeSetEvent(
        (UINT)msFrameRate,     // delay (ms)
        TIMER_REZ,             // resolution (ms)
        Tmr_handler,           // callback function
        nModelUpdatesPerFrame, // user data
        TIME_PERIODIC);
}

//-------------------------------------------------------------------------------------------------
static BOOL Wincore_OnCreate (HWND hwnd, CREATESTRUCT FAR *lpCreateStruct)
{
  OutputDebugString ("Creation\n");

  /* Splash screen */
  DialogBoxParam (hInstance, MAKEINTRESOURCE (AboutBox), hwnd, (DLGPROC)About, TRUE);

  return TRUE;
}

//-------------------------------------------------------------------------------------------------
static void Wincore_OnDestroy (HWND hwnd)
{
  extern void CleanupBGI (void);

  /* Don't process any more timer messages */
  isPaused = TRUE;

  timeKillEvent (wTimerID);
  timeEndPeriod (TIMER_REZ);

  CleanupBGI();

  PostQuitMessage (0);
}

//----------------------------------------------------------------------------
static void Wincore_OnClose(HWND hwnd)
{
  DestroyWindow (hwnd);
}

//-------------------------------------------------------------------------------------------------
static void Wincore_OnCommand (HWND hwnd, int id, HWND hwndCtl, UINT codeNotify)
{
  switch (id)  {
    case IDM_ABOUT:
      DialogBoxParam (hInstance, MAKEINTRESOURCE (AboutBox), hwnd, (DLGPROC)About, FALSE);
      break;
    case IDM_RESTART:
    {
      extern void	InitFlight(void);
      InitFlight();
      break;
    }
    case IDM_EXIT:
      DestroyWindow (hwnd);
      break;
    default:
      FORWARD_WM_COMMAND(hwnd, id, hwndCtl, codeNotify, Wincore_DefProc);
  }
}

//-------------------------------------------------------------------------------------------------
static LRESULT CALLBACK About (HWND hDlg, UINT message, WPARAM wParam, LPARAM lParam)
{
  static BOOL isSplash;

  switch (message) {
    case WM_INITDIALOG:
      isSplash = (BOOL)lParam;
      if (isSplash)  {
        SetTimer(hDlg, IDT_TIMER_SPLASH, SPLASH_DELAY, NULL);
      }
      return (TRUE);
    case WM_TIMER:
      KillTimer (hDlg, IDT_TIMER_SPLASH);
      EndDialog(hDlg, TRUE);
      return (TRUE);
    case WM_COMMAND:
      if (LOWORD(wParam) == IDOK || LOWORD(wParam) == IDCANCEL) {
        EndDialog(hDlg, TRUE);
        return (TRUE);
      }
      break;
  }
  return FALSE;
}

//-------------------------------------------------------------------------------------------------
 
