/*-------------------------------------------------------------------------------------------------
  File Name:  winjoy.c

  Purpose:    RCsimWin - 
              Windows shim for RCsim joystick support.

  Author   :  David B. Ewing
              codetoad@rocketmail.com

  History:    07/03/98  Created

             Copyright (c) 1998  David B. Ewing 
    All rights reserved.  May be copied and distributed freely,
    provided this copyright notice is preserved.

    RCsim - Copyright (c) 1995 Mikko Oksalahti and Harri Rautio
-------------------------------------------------------------------------------------------------*/

#include <windows.h>
#include <stdio.h>
#include "joystick.h"

struct joystick JoyStick;

/* Calibration constants determined by init_joystick() */
static JOYCAPS jc;
static float joyXscale = 1.0F;
static float joyYscale = 1.0F;
static float joyZscale = 1.0F;

/* Default ranges set forth by Toad (probably matched his joystick...) */
UINT xFullScale = 80;
UINT yFullScale = 80;
UINT const zFullScale = 60;

LOGICAL   read_joystick(void)
{
  JOYINFO ji;

  if (joyGetPos(JOYSTICKID1, &ji) != JOYERR_NOERROR)  {
    return JOY_ERROR;
  }

  JoyStick.switch_0 = !(ji.wButtons & JOY_BUTTON1);
  JoyStick.switch_1 = !(ji.wButtons & JOY_BUTTON2);
  JoyStick.switch_2 = !(ji.wButtons & JOY_BUTTON3);
  JoyStick.switch_3 = !(ji.wButtons & JOY_BUTTON4);
  JoyStick.pos_Ax   = (int)((ji.wXpos - jc.wXmin) * joyXscale);
  JoyStick.pos_Ay   = (int)((ji.wYpos - jc.wYmin) * joyYscale);
  JoyStick.pos_Bx   = 0;
  JoyStick.pos_By   = (int)((ji.wZpos - jc.wZmin) * joyZscale);

  return SUCCESS;
}

void init_joystick (void)
  /* Call this first, unless you want the raw stick values */
{
  extern int throttleswitch;

  joyGetDevCaps (JOYSTICKID1, &jc, sizeof (JOYCAPS));

  joyXscale = (float)xFullScale / (jc.wXmax - jc.wXmin);
  joyYscale = (float)yFullScale / (jc.wYmax - jc.wYmin);
  joyZscale = (float)zFullScale / (jc.wZmax - jc.wZmin);

  /* Detect joystick throttle */
  throttleswitch = jc.wNumAxes > 2;
}

#if 0

int main (void)
  /* Test program (console mode) */
{
  if(read_joystick() != SUCCESS)  {
    printf ("\nJoystick not found.\n");
    return 0;
  }

  /* Remove this to see raw numbers */
  init_joystick();

  printf ("\nJoystick Test  (^C to exit)\n\n");
  while (1)  {
    if(!read_joystick())  {
			printf("%3u %3u %3u %3u  ",
				JoyStick.pos_Ax,		/* left/right,	0..75 */
				JoyStick.pos_Ay,		/* up/down,		5..74 */
				JoyStick.pos_Bx,		/* increases with more left */
				JoyStick.pos_By);		/* Throttle full/off  60..5 */
      putchar(JoyStick.switch_0 ? '0' : ' ');   /* trigger */
      putchar(JoyStick.switch_1 ? '1' : ' ');   /* bomb */
      putchar(JoyStick.switch_2 ? '2' : ' ');   /* bomb */
      putchar(JoyStick.switch_3 ? '3' : ' ');   /* nothing */
      putchar('\r');
		}
  }

  return 0;
}

#endif